/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "interface.h"

static const struct luaL_reg lobj [] =
{
	{"version", lobj_version},
	{"convert", lobj_convert},
	{"getCharset", lobj_get_charset},
	{NULL, NULL}
};

static const struct luaL_reg timeout [] =
{
	{"add", lobj_timeout_add},
	{"idleAdd", lobj_timeout_idle_add},
	{"remove", lobj_timeout_remove},
	{NULL, NULL}
};

static const struct luaL_reg object [] =
{	
	{"set", lobj_object_set},
	{"get", lobj_object_get},
	{"connect", lobj_object_connect},
	{"disconnect", lobj_object_disconnect},
	{"block", lobj_object_block},
	{"unblock", lobj_object_unblock},
	{"ref", lobj_object_ref},
	{"unref", lobj_object_unref},
	{"notify", lobj_object_notify},
	{"thawNotify", lobj_object_thaw_notify},
	{"freezeNotify", lobj_object_freeze_notify},
	{NULL, NULL}
};

static const struct luaL_reg mainloop [] =
{	
	{"new", lobj_main_loop_new},
	{"iterate", lobj_main_loop_iterate},
	{"pending", lobj_main_loop_pending},
	{"quit", lobj_main_loop_quit},
	{"run", lobj_main_loop_run},
	{NULL, NULL}
};

static const struct luaL_reg checksum [] =
{
	{"getMD5", lobj_checksum_get_md5},
	{"getSHA1", lobj_checksum_get_sha1},
	{"getSHA256", lobj_checksum_get_sha256},
	{NULL, NULL}
};

static const struct luaL_reg timer [] =
{
	{"new", lobj_timer_new},
	{"start", lobj_timer_start},
	{"stop", lobj_timer_stop},
	{"continue", lobj_timer_continue},
	{"elapsed", lobj_timer_elapsed},
	{NULL, NULL}
};

static const struct luaL_reg environment [] =
{
	{"getUserName", lobj_environment_get_user_name},
	{"getRealName", lobj_environment_get_real_name},
	{"getUserDataDir", lobj_environment_get_user_data_dir},
	{"getUserCacheDir", lobj_environment_get_user_cache_dir},
	{"getUserConfigDir", lobj_environment_get_user_config_dir},
	{"getTmpDir", lobj_environment_get_tmp_dir},
	{"getCurrentDir", lobj_environment_get_current_dir},
	{"getHomeDir", lobj_environment_get_home_dir},
	{"getHostName", lobj_environment_get_host_name},
	{NULL, NULL}
};

static const struct luaL_reg path [] =
{
	{"isAbsolute", lobj_path_is_absolute},
	{"skipRoot", lobj_path_skip_root},
	{"getBasename", lobj_path_get_basename},
	{"getDirname", lobj_path_get_dirname},
	{"findProgram", lobj_path_find_program},
	{NULL, NULL}
};

static const struct luaL_reg date [] =
{
	{"new", lobj_date_new},
	{"get", lobj_date_get},
	{"set", lobj_date_set},
	{"valid", lobj_date_valid},
	{"compare", lobj_date_compare},
	{NULL, NULL}
};

static const struct luaL_reg rand [] =
{
	{"new", lobj_rand_new},
	{"setSeed", lobj_rand_set_seed},
	{"int", lobj_rand_int},
	{"double", lobj_rand_double},
	{"boolean", lobj_rand_boolean},
	{NULL, NULL}
};

static const struct luaL_reg type [] =
{
	{"fromName", lobj_type_from_name},
	{"fromInstance", lobj_type_from_instance},
	{"nameFromType", lobj_type_name_from_type},
	{NULL, NULL}
};

static const struct luaL_reg base64 [] =
{
	{"encode", lobj_base64_encode},
	{"decode", lobj_base64_decode},
	{NULL, NULL}
};

static const struct luaL_reg utf8 [] =
{
	{"normalize", lobj_utf8_normalize},
	{"strup", lobj_utf8_strup},
	{"strdown", lobj_utf8_strdown},
	{"validate", lobj_utf8_validate},
	{"strlen", lobj_utf8_strlen},
	{"collate", lobj_utf8_collate},
	{"collateKey", lobj_utf8_collate_key},
	{"collateKeyForFilename", lobj_utf8_collate_key_for_filename},
	{"fromFilename", lobj_utf8_from_filename},
	{"toFilename", lobj_utf8_to_filename},
	{"casefold", lobj_utf8_casefold},
	{NULL, NULL}
};

int luaopen__lobj(lua_State *L)
{
	/* Initialization */
	g_type_init();
	
	/* Register the classes */
	luaL_register(L, LIB_NAME, lobj);
	luaL_register(L, "lobj.Object", object);
	luaL_register(L, "lobj.MainLoop", mainloop);
	luaL_register(L, "lobj.Timeout", timeout);
	luaL_register(L, "lobj.Checksum", checksum);
	luaL_register(L, "lobj.Timer", timer);
	luaL_register(L, "lobj.Environment", environment);
	luaL_register(L, "lobj.Path", path);
	luaL_register(L, "lobj.Date", date);
	luaL_register(L, "lobj.Rand", rand);
	luaL_register(L, "lobj.Type", type);
	luaL_register(L, "lobj.Base64", base64);
	luaL_register(L, "lobj.UTF8", utf8);
	
	/* Push some functions out of the tables for faster access */
	lua_register(L, "__lobjObjectNew", lobj_object_new); 
	lua_register(L, "__lobjStructNew", lobj_struct_new); 
	lua_register(L, "__lobjBoxedNew", lobj_boxed_new); 
	lua_register(L, "__lobjGc", lobj_object_gc); 
	lua_register(L, "__lobjValuePush", lobj_value_push); 
	lua_register(L, "__lobjValueSet", lobj_value_set); 
	lua_register(L, "__lobjStructGc", lobj_struct_gc); 
	lua_register(L, "__lobjStructTostring", lobj_struct_to_string); 
	lua_register(L, "__lobjTostring", lobj_object_to_string); 
	lua_register(L, "__lobjEq", lobj_object_eq);
	lua_register(L, "__lobjClosureNew", lobj_closure_new);
	lua_register(L, "__lobjGstrv2table", lobj_gstrv2table);
	lua_register(L, "__lobjTable2gstrv", lobj_table2gstrv);
	lua_register(L, "__lobjGSList2table", lobj_gslist2table);
	lua_register(L, "__lobjGList2table", lobj_glist2table);
	
	return 1;
}
