/*
	This file is part of lobj.

	lobj is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	lobj is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with lobj.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2008 Lucas Hermann Negri
*/

#include "string.h"

int lobj_utf8_validate(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gboolean res = g_utf8_validate(str, len, NULL);
	lua_pushboolean(L, res);
	
	return 1;
}

int lobj_utf8_strup(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_strup(str, len);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_strdown(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_strdown(str, len);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_casefold(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_casefold(str, len);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}


int lobj_utf8_normalize(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TNUMBER);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_normalize(str, len, (GNormalizeMode)lua_tointeger(L, 2));
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_strlen(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	glong res = g_utf8_strlen(str, len);
	lua_pushinteger(L, res);
	
	return 1;
}

int lobj_utf8_collate(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TSTRING);
	
	gint res = g_utf8_collate(lua_tostring(L, 1), lua_tostring(L, 2));
	lua_pushinteger(L, res);
	
	return 1;
}

int lobj_utf8_collate_key(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_collate_key(str, len);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_collate_key_for_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_utf8_collate_key_for_filename(str, len);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_from_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_filename_to_utf8(str, len, NULL, NULL, NULL);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_utf8_to_filename(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	size_t len;
	const char* str = lua_tolstring(L, 1, &len);
	
	gchar* res = g_filename_from_utf8(str, len, NULL, NULL, NULL);
	lua_pushstring(L, res);
	g_free(res);
	
	return 1;
}

int lobj_convert(lua_State* L)
{
	luaL_checktype(L, 1, LUA_TSTRING);
	luaL_checktype(L, 2, LUA_TSTRING);
	luaL_checktype(L, 3, LUA_TSTRING);
	
	size_t len, bread, bwrite;
	const char* str = lua_tolstring(L, 1, &len);
	GError* errors = NULL;
	
	char* res = g_convert(str, len, lua_tostring(L, 2), lua_tostring(L, 3), 
		&bread, &bwrite, &errors);
		
	if(errors)
	{
		lua_pushboolean(L, FALSE);
		lua_pushinteger(L, errors->code);
		lua_pushstring(L, errors->message);
		g_error_free(errors);
		if(res) g_free(res);
		return 3;
	}
	
	lua_pushlstring(L, res, bwrite);
	g_free(res);
	
	return 1;
}

int lobj_get_charset(lua_State* L)
{
	const char* charset;
	gboolean res = g_get_charset(&charset);
	
	lua_pushboolean(L, res);
	lua_pushstring(L, charset);
	return 2;
}
